/*
Copyright (c) 2019-2022 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>
Tim Tkachev, <tvtkachev@basistech.ru>
Nikita Sorokin, <nesorokin@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package disks

import (
	"context"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/disks"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
)

func utilityDiskListDeletedCheckPresence(ctx context.Context, d *schema.ResourceData, m interface{}) (*disks.ListDisks, error) {
	c := m.(*controller.ControllerCfg)
	req := disks.ListDeletedRequest{}

	if by_id, ok := d.GetOk("by_id"); ok {
		req.ByID = uint64(by_id.(int))
	}
	if name, ok := d.GetOk("name"); ok {
		req.Name = name.(string)
	}
	if account_name, ok := d.GetOk("account_name"); ok {
		req.AccountName = account_name.(string)
	}
	if disk_max_size, ok := d.GetOk("disk_max_size"); ok {
		req.DiskMaxSize = int64(disk_max_size.(int))
	}
	if shared, ok := d.GetOk("shared"); ok {
		req.Shared = shared.(bool)
	}
	if account_id, ok := d.GetOk("account_id"); ok {
		req.AccountID = uint64(account_id.(int))
	}
	if typev, ok := d.GetOk("type"); ok {
		req.Type = typev.(string)
	}
	if sortBy, ok := d.GetOk("sort_by"); ok {
		req.SortBy = sortBy.(string)
	}
	if page, ok := d.GetOk("page"); ok {
		req.Page = uint64(page.(int))
	}
	if size, ok := d.GetOk("size"); ok {
		req.Size = uint64(size.(int))
	}

	log.Debugf("utilityDiskListDeletedCheckPresence: load disk list deleted")
	diskList, err := c.CloudBroker().Disks().ListDeleted(ctx, req)
	if err != nil {
		return nil, err
	}

	return diskList, nil
}
