/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package lb

import (
	"context"
	"fmt"
	"strconv"
	"strings"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/lb"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
)

func utilityLBBackendServerCheckPresence(ctx context.Context, d *schema.ResourceData, m interface{}) (*lb.ItemServer, error) {
	c := m.(*controller.ControllerCfg)
	req := lb.GetRequest{}

	bName := d.Get("backend_name").(string)
	sName := d.Get("name").(string)

	if d.Id() != "" {
		parameters := strings.Split(d.Id(), "#")
		lbId, _ := strconv.ParseUint(parameters[0], 10, 64)
		req.LBID = lbId
		bName = parameters[1]
		sName = parameters[2]
	} else {
		req.LBID = uint64(d.Get("lb_id").(int))
	}

	foundLB, err := c.CloudBroker().LB().Get(ctx, req)
	if err != nil {
		return nil, err
	}

	backend := &lb.ItemBackend{}
	backends := foundLB.Backends
	for i, b := range backends {
		if b.Name == bName {
			backend = &backends[i]
			break
		}
	}

	if backend.Name == "" {
		return nil, fmt.Errorf("can not find backend with name: %s for lb: %d", bName, foundLB.ID)
	}

	for _, s := range backend.Servers {
		if s.Name == sName {
			return &s, nil
		}
	}

	return nil, fmt.Errorf("can not find server with name: %s for backend: %s for lb: %d", sName, bName, foundLB.ID)
}
