Функция доступна в провайдере версии 4.5.1 и выше.

_Data source_ функция **decort_cb_rg_list_lb** служит для получения информации о списке lb ресурсной группы.

Ресурсная группа в облачной платформе DECORT это способ группирования _compute_-ов (например, по функциональному признаку или принадлежности к одному и тому же проекту). Ресурсную группу можно рассматривать как небольшой персональный дата-центр, в котором размещаются один или несколько серверов и виртуальных сетевых сегментов (ViNS).

## Аргументы
_Data source_ функция **decort_cb_rg_list_lb** принимает следующие аргументы:

| Аргумент | Тип    | Обязательный | Описание |
| --- |--------| --- | --- |
| rg_id | int    | :heavy_check_mark: | Идентификатор ресурсной группы |
| by_id | int    | :x: | Фильтр по ID балансировщика |
| name | string | :x: | Фильтр по имени балансировщика |
| account_id | int    | :x: | Фильтр по ID аккаунта |
| tech_status | string | :x: | Фильтр по тех. статусу |
| status | string | :x: | Фильтр по статусу |
| front_ip | string | :x: | Фильтр по IP фронтенда |
| back_ip | string | :x: | Фильтр по IP бэкенда |
| page | int    | :x: | Номер страницы |
| size | int    | :x: | Размер страницы |

Пользователь облачной платформы, от лица которого происходит обращение к данной функции, должен иметь доступ к указанному `account` и ресурсной группе. В противном случае возникнет ошибка доступа.

Пользователь облачной платформы задается на стадии инициализации Terraform провайдера DECORT (см. [подробности](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.5.2/04.02-Инициализация-Terraform-провайдера-DECORT.md)).

## Возвращаемые значения
В случае успешного выполнения _data source_ функция **decort_cb_rg_list_lb** возвращает в указанную при вызове переменную, в поле _items_, массив, каждый элемент которого состоит из следующих параметров:

| Параметр       | Тип                                                     | Описание                                     |
|----------------|---------------------------------------------------------|----------------------------------------------|
| ha_mode        | bool                                                    | Флаг, показывающий, включен ли режим haproxy |
| acl            | []Struct [см. ниже](#описание-структуры-acl)            | Cписок Acces Control                         |
| backends       | []Struct{} [см. ниже](#описание-структуры-backends)     | Список бекендов                              |
| created_by     | string                                                  | Аккаунт, создавший балансировщик             |
| created_time   | int                                                     | Время создания                               |
| deleted_by     | string                                                  | Кем балансировщик был удален                 |
| deleted_by     | int                                                     | Время удаления                               |
| desc           | string                                                  | Описание балансировщика                      |
| dp_api_user    | string                                                  | Имя пользователя для работы                  |
| extnet_id      | int                                                     | ID внешней сети                              |
| frontends      | []Struct{} [см. ниже](#описание-структуры-frontends)    | Список фронтендов                            |
| gid            | int                                                     | Grid ID балансировщика                       |
| guid           | int                                                     | ID балансировщика                            |
| id             | int                                                     | ID балансировщика                            |
| image_id       | int                                                     | ID образа                                    |
| milestones     | int                                                     | Вехи                                         |
| name           | string                                                  | Имя балансировщика                           |
| primary_node   | Struct{} [см. ниже](#описание-структуры-primary_node)   | Главный узел                                 |
| rg_name        | string                                                  | Название ресурсной группы                    |
| secondary_node | Struct{} [см. ниже](#описание-структуры-secondary_node) | Второй узел                                  |
| status         | string                                                  | Статус                                       |
| tech_status    | string                                                  | Технический статус                           |
| updated_by | string | Кем был обновлен балансировщик               |
| updated_time | int | Время последнего обновления балансировщик          |
| vins_id        | int                                                     | ID виртуальной сети                          |

### Описание структуры primary_node
| Параметр    | Тип    | Описание              |
|-------------|--------|-----------------------|
| backend_ip  | string | IP адрес бекенда      |
| compute_id  | int    | ID виртуальной машины |
| frontend_ip | string | IP адрес фронтенда    |
| guid        | string | ID ресурса            |
| mgmt_ip     | string | IP адрес              |
| network_id  | int    | ID сети               |

### Описание структуры secondary_node
| Параметр    | Тип    | Описание              |
|-------------|--------|-----------------------|
| backend_ip  | string | IP адрес бекенда      |
| compute_id  | int    | ID виртуальной машины |
| frontend_ip | string | IP адрес фронтенда    |
| guid        | string | ID ресурса            |
| mgmt_ip     | string | IP адрес              |
| network_id  | int    | ID сети               |

### Описание структуры frontends
| Параметр | Тип                                                                                                                                                                                                             | Описание                                   |
|----------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------|
| backend  | string                                                                                                                                                                                                          | Имя бекенда, к которому подключен фронтенд |
| guid     | string                                                                                                                                                                                                          | ID ресурса                                 |
| name     | string                                                                                                                                                                                                          | Имя фронтенда                              |
| bindings | []Struct{} [см. ниже](#описание-структуры-bindings) | Привязки фронтенда                         |

### Описание структуры bindings
| Параметр | Тип    | Описание                            |
|----------|--------|-------------------------------------|
| address  | string | Адрес, к которому привязан фронтенд |
| name     | string | Имя привязки                        |
| port     | int    | Порт привязки                       |
| guid     | string | ID ресурса                          |

### Описание структуры acl
| Параметр | Тип | Описание |
| --- | --- | --- |
| explicit | bool | Явно ли указан доступ |
| guid | string | guid Ресурса |
| right | string | Права на ресурс |
| status | string | Статус ресурса |
| type | string | Тип ресурса |
| user_group_id | string | ID user group |

### Описание структуры backends
| Параметр | Тип                                                             | Описание |
| --- |-----------------------------------------------------------------| --- |
| algorithm | string                                                          | Алгоритм работы бекенда |
| guid | string                                                          | ID ресурса |
| name | string                                                          | Имя бекенда |
| server_default_settings | Struct{} [см. ниже](#описание-структуры-server_settings) | Описание настроек серверов бекенда по умолчанию |
| servers | []Struct{} [см. ниже](#описание-структуры-servers)              | Список серверов |

### Описание структуры servers
| Параметр | Тип                                                        | Описание |
| --- |------------------------------------------------------------| --- |
| address | string                                                     | Адрес сервера |
| check | string                                                     | Включены ли проверки |
| guid | string                                                     | ID ресурса |
| name | string                                                     | Имя сервера |
| port | int                                                        | Порт сервера |
| server_settings | []Struct{} [см. ниже](#описание-структуры-server_settings) | Настройки сервера |

### Описание структуры привязки фронтенда
| Параметр | Тип | Описание |
| --- | --- | --- |
| address | string | Адрес, к которому привязан фронтенд |
| name | string | Имя привязки |
| port | int | Порт привязки |

### Описание структуры server_settings
| Параметр   | Тип | Описание |
|------------| --- | --- |
| down_inter | int | Интервал между проверками на недоступность сервера |
| guid       | string | ID ресурса |
| inter      | int | Интервал между проверками на доступность сервера |
| rise       | int | Кол-во попыток, после которых, сервер будет вновь включен в схему балансирования |
| fall       | int | Кол-во неудачных попыток, после которых, сервер получит статус "unavailable" |
| slow_start | int | Интервал, в мс, в через который сервер получит статус "available" |
| max_conn   | int | Кол-во соединений к северу, при достижении этого кол-ва, сервера выходит из схемы балансирования |
| max_queue  | int | Кол-во соедений в очереди к серверу |
| weight     | int | Вес сервера для работы алгоритма балансирования по весу |

## Пример использования
Пример вызова _data source_ функции **decort_cb_rg_list_lb**:

```terraform
data "decort_cb_rg_list_lb" "list_lb" {
  rg_id = 123
  #by_id = 100
  #тип - строка
  #name = "test"
  #account_id = 11111
  #tech_status = "STOPPED"
  #status = "ENABLED"
  #front_ip = "ENABLED"
  #back_ip = "ENABLED"
  #page = 1
  #size = 1
}
# Once the above directive completes, resource group list of load balancers will be accessible as
# data.decort_cb_rg_list_lb.list_lb.items
```

Данный пример подразумевает, что ранее по тексту tf-файла выполнена корректная инициализация провайдера (подробнее об инициализации Terraform провайдера DECORT см. соответствующий [раздел](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.5.2/04.02-Инициализация-Terraform-провайдера-DECORT.md) и [пример](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.5.2/02.-Пример-работы.md)). 

После успешного завершения такого вызова **decort_cb_rg_list_lb** в переменной `data.decort_cb_rg_list_lb.list_lb.items` будут сохранены возвращаемые значения.
