Функция доступна в провайдере версии 4.5.1 и выше.

_Data source_ функция **decort_cb_account_list_deleted** служит для получения информации об удаленных аккаунтах, которые можно восстановить в дальнейшем. В данный список попадают аккаунты, удаленные с флагом permanently, выставленным в false.

## Аргументы

_Data source_ функция **decort_cb_account_list_deleted** принимает следующие аргументы:

| Аргумент | Тип | Обязательный | Описание |
| --- | --- | --- | --- |
| by_id | int | :x: | Фильтр по ID аккаунта |
| name | string | :x: | Фильтр по имени аккаунта      |
| acl | string | :x: | Фильтр по ACL |
| page | int | :x: | Номер страниц         |
| size | int | :x: | Размер страницы       |

## Возвращаемые значения

В случае успешного выполнения _data source_ функция **decort_cb_account_list_deleted** возвращает в указанную при вызове переменную, в поле _items_ массив, каждый элемент которого состоит из следующих параметров:

| Параметр | Тип | Описание                                                |
| --- | --- |---------------------------------------------------------|
|dc_location|string| dc location                                             |
| ckey | string | CKey                                                    |
| meta | []string | Мета информация об аккаунте                             |
| acl | []Struct [см. ниже](#описание-структуры-acl)| Cписок Acces Control                                    |
| company | string | Название компании, которой принадлежит аккаунт          |
| companyurl | string | url-адрес компани                                       |
| cpu_allocation_parameter | string | Параметр выделения CPU                                  |
| cpu_allocation_ratio | float | Отношение выделения CPU                                 |
| created_by | string | Пользователь - создателя аккаунта                       |
| created_time | int | Время создания аккаунта                                 |
| deactivation_time | float | Время деактивации аккаунта                              |
| deleted_by | string | Пользователь, удаливший аккаунт                         |
| deleted_time | int | Время удаления аккаунта                                 |
| displayname | string | Отображаемое имя аккаунта                               |
| guid | int | Внутренний guid аккаунта                                |
| account_id | int | ID аккаунта                                             |
| account_name | string | Название аккаунта                                       |
| resource_limits | Struct{} [см. ниже](#описание-структуры-resource_limits) | Лимиты ресурсов                                         |
| resource_types           | []string                                                 | Типы ресурсов                                           |
| send_access_emails | bool | Флаг, отправлять ли на указанную почту письмо о доступе |
| status | string | Статус аккаунта                                         |
| uniq_pools               | []string                                                 | Список пулов                                            |
| updated_time | int | Время последнего обновления                             |
| version | int | Версия аккаунта                                         |
| vins | []int | Список id виртуальных сетей                             |

### Описание структуры acl
| Параметр | Тип | Описание |
| --- | --- | --- |
| explicit | bool | Явно ли указан доступ |
| guid | string | guid Ресурса |
| right | string | Права на ресурс |
| status | string | Статус ресурса |
| type | string | Тип ресурса |
| user_group_id | string | ID user group |

### Описание структуры resource_limits
| Параметр | Тип | Описание |
| --- | --- | --- |
| cu_c | float | Количество ядер CPU |
| cu_d | float | Размер диска |
| cu_dm | float | Использованный размер хранилища виртуального диска | 
| cu_i | float | Количество публичных IP адресов |
| cu_m | float | размер RAM |
| cu_np | float | Хранилище траффика |
| gpu_units | float | количество GPU ядер |

## Пример использования

Пример вызова _data source_ функции **decort_cb_account_list_deleted**:

```terraform
data "decort_cb_account_list_deleted" "adl" {
  #фильтр по id аккаунта
  #опциональный параметр
  #тип - число
  #by_id = 100

  #фильтр по имени аккаунта
  #опциональный параметр
  #тип - строка
  #name = "test"

  #фильтр по ACL
  #опциональный параметр
  #тип - строка
  #acl = "test"
  
  #номер страницы для отображения
  #опциональный параметр
  #тип - число
  #если не задан - выводятся все доступные данные
  #page = 2

  #размер страницы
  #опциональный параметр
  #тип - число
  #если не задан - выводятся все доступные данные
  #size = 3
}
# once the above directive completes, account deleted list will be accessible
# as data.decort_cb_account_list_deleted.adl.items
```

Данный пример подразумевает, что ранее по тексту tf-файла выполнена корректная инициализация провайдера (подробнее об инициализации Terraform провайдера DECORT см. соответствующий [раздел](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.5.3/04.02-Инициализация-Terraform-провайдера-DECORT.md)).

После успешного завершения такого вызова **decort_cb_account_list_deleted** в переменной `data.decort_cb_account_list_deleted.adl.items` будут сохранены возвращаемые значения.
