Функция доступна в провайдере версии 4.5.1 и выше.

_Data source_ функция **decort_cb_account_rg_list** служит для получения информации о ресурсных группах, принадлежащих аккаунту.

## Аргументы

_Data source_ функция **decort_cb_account_rg_list** принимает следующие аргументы:

| Аргумент | Тип | Обязательный | Описание |
| --- | --- | --- | --- |
| account_id | int | :heavy_check_mark: | ID аккаунта |
| rg_id | int | :x: | ID ресурсной группы для фильтрации результата |
| page | int | :x: | Номер страниц |
| size | int | :x: | Размер страницы |
| name | string | :x: | Фильтр по имени ресурсной группы |
| vins_id | int | :x: | Фильтр по ID ViNS |
| vm_id | int | :x: | Фильтр по ID вирт. машины |
| status | string | :x: | Фильтр по статусу |
| sort_by | string | :x: | Фильтр по одному из поддерживаемых полей, формат: +или- название поля |

## Возвращаемые значения

В случае успешного выполнения _data source_ функция **decort_cb_account_rg_list** возвращает в указанную при вызове переменную, в поле _items_ массив, каждый элемент которого состоит из следующих параметров:

| Параметр | Тип | Описание |
| --- | --- | --- |
| computes | Struct{<br/>started int<br/>stopped int <br/>} | Информация о вычислительных мощностях<br/>Кол-во запущенных<br/>Кол-во остановленных<br/>&nbsp;<br/> |
| resources | Struct{} [см. ниже](#описание-структуры-resources) | Информация о занимаемых ресурсах |
| created_by | string | Информация о создателе группы |
| created_time | int | Время создания группы |
| deleted_by | string | Информация о том, кто удалил группу |
| deleted_time | int | Время удаления группы |
| rg_id | int | Идентификатор ресурсной группы |
| rg_name | string | Название ресурсной группы |
| milestones | int | Вехи |
| status | string | Статус группы |
| updated_by | string | Кем группа была обновлена в последний раз |
| updated_time | int | Время последнего обновления |
| vinses | int | Кол-во виртуальных сетей |

### Описание структуры resources
| Параметр | Тип | Описание |
| --- | --- | --- |
| consumed | Struct{} [см. ниже](#описание-структуры-consumed) | Информация о текуших ресурсах |
| limits | Struct{} [см. ниже](#описание-структуры-limits) | Информация о лимитах аккаунта |
| reserved | Struct{} [см. ниже](#описание-структуры-reserved) | Информация о зарезверированных аккаунтом ресурсах |

### Описание структуры consumed
| Параметр | Тип | Описание |
| --- | --- | --- |
| cpu | int | Количество CPU |
| disksize | float | Размер диска |
| disksizemax | int | Максимальный размер диск |
| extips | int | Внешние IPs |
| exttraffic | int | Внешний Traffic |
| gpu | int | Количество GPU |
| ram | int | Количество RAM |
| seps | []Struct{} [см. ниже](#описание-структуры-seps) | Информация о seps |

### Описание структуры limits
| Параметр | Тип | Описание |
| --- | --- | --- |
| cpu | int | Количество CPU |
| disksize | int | Размер диска |
| disksizemax | int | Максимальный размер диск |
| extips | int | Внешние IPs |
| exttraffic | int | Внешний Traffic |
| gpu | int | Количество GPU |
| ram | int | Количество RAM |
| seps | []int | Список id seps |

### Описание структуры reserved
| Параметр | Тип | Описание |
| --- | --- | --- |
| cpu | int | Количество CPU |
| disksize | int | Размер диска |
| disksizemax | int | Максимальный размер диск |
| extips | int | Внешние IPs |
| exttraffic | int | Внешний Traffic |
| gpu | int | Количество GPU |
| ram | int | Количество RAM |
| seps | []Struct{} [см. ниже](#описание-структуры-seps) | Информация о seps |

### Описание структуры seps
| Параметр | Тип | Описание |
| --- | --- | --- |
| sep_id | string | ID SEP. SEP это способ идентифицировать конкретную систему хранения, на ресурсах которой размещён данный диск |
| data_name | string | Имя пула информации внутри SEP'а |
| disk_size | float | Количество занятого места |
| disk_size_max | float | Размер диска |

## Пример использования

Пример вызова _data source_ функции **decort_cb_account_rg_list**:

```terraform
data "decort_cb_account_rg_list" "argl" {
  #id аккаунта
  #обязательный параметр
  #тип - целое число
  account_id = 66666

  #сортировка по одному из поддерживаемых полей
  #опциональный параметр
  #формат - "+поле" по возрастанию / "-поле" по убыванию
  #тип - строка
  #sort_by = "+name"

  #номер страницы для отображения
  #опциональный параметр
  #если не задан - выводятся все доступные данные
  #тип - целое число
  #page = 2

  #размер страницы
  #опциональный параметр
  #если не задан - выводятся все доступные данные
  #тип - целое число
  #size = 3

  #фильтр по id ресурсной группы
  #опциональный параметр 
  #тип - целое число
  #rg_id = 11111

  #фильтр по имени ресурсной группы
  #опциональный параметр
  #тип - строка
  #name = "test"

  #фильтр по id vins
  #опциональный параметр
  #тип - целое число
  #vins_id = 100

  #фильтр по id compute
  #опциональный параметр
  #тип - целое число
  #vm_id = 100

  #фильтр по статусу
  #опциональный параметр
  #тип - строка
  #status = "CREATED"
}
```

Данный пример подразумевает, что ранее по тексту tf-файла выполнена корректная инициализация провайдера (подробнее об инициализации Terraform провайдера DECORT см. соответствующий [раздел](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.6.2/04.02-Инициализация-Terraform-провайдера-DECORT.md)).

После успешного завершения такого вызова **decort_cb_account_rg_list** в переменной `data.decort_cb_account_rg_list.argl.items` будут сохранены возвращаемые значения.
