_Data source_ функция **decort_rg_list_vins** служит для получения информации о списке vins ресурсной группы.

Ресурсная группа в облачной платформе DECORT это способ группирования _compute_-ов (например, по функциональному признаку или принадлежности к одному и тому же проекту). Ресурсную группу можно рассматривать как небольшой персональный дата-центр, в котором размещаются один или несколько серверов и виртуальных сетевых сегментов (ViNS).

Экземпляры _compute_ (виртуальные серверы) всегда создаются в принадлежности к какой-либо ресурсной группе. Как следствие, для создания экземпляра _compute_ помимо других параметров требуется указать идентификатор целевой ресурсной группы. Для получения такого идентификатора можно воспользоваться рассматриваемой здесь _data source_ функцией **decort_rg_list_vins**.

## Аргументы
_Data source_ функция **decort_rg_list_vins** принимает следующие аргументы:

| Аргумент | Тип | Обязательный | Описание |
| --- | --- | --- | --- |
| rg_id | int | :heavy_check_mark: | Идентификатор ресурсной группы |
| name | string | :x: | Фильтр по имени ViNS |
| account_id | int | :x: | Фильтр по ID аккаунта |
| ext_ip | string | :x: | Фильтр по внешнему IP |
| vins_id | int | :x: | Фильтр по ID ViNS |
| sort_by | string | :x: | Фильтр по одному из поддерживаемых полей, формат: +или- название поля |
| page | int | :x: | Номер страницы |
| size | int | :x: | Размер страницы |

Пользователь облачной платформы, от лица которого происходит обращение к данной функции, должен иметь доступ к указанному `account` и ресурсной группе. В противном случае возникнет ошибка доступа.

Пользователь облачной платформы задается на стадии инициализации Terraform провайдера DECORT (см. [подробности](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.7.1/04.02-Инициализация-Terraform-провайдера-DECORT.md)).

## Возвращаемые значения
В случае успешного выполнения _data source_ функция **decort_rg_list_vins** возвращает в указанную при вызове переменную, в поле _items_, массив, каждый элемент которого состоит из следующих параметров:
| Параметр | Тип | Описание |
| --- | --- | --- |
| account_id | int | Уникальный идентификатор подписчика-владельца ViNS. |
| account_name | string | Имя подписчика ("account"), которому принадлежит ViNS. |
| computes | []int | Список компьютов |
| created_by | int | Кем создан ресурс |
| created_time | int | Время создания |
| deleted_by | string | Кем удален |
| deleted_time | int | Время удаления |
| external_ip | string | Внешний ip |
| extnet_id | int | ID внешней сети |
| free_ips | int | Кол-во свободных ip-адресов |
| id | int | идентификатор ресурса |
| name | string | Имя ресурса |
| network | string | Сеть |
| pri_vnf_dev_id | int | ID vnf dev |
| rg_name | string | Имя ресурсной группы |
| status | string | Статус |
| updated_by | string | Кем обновлен ресрус |
| updated_time | int | Время обнавления ресурса |

## Пример использования
Пример вызова _data source_ функции **decort_rg_list_vins**:

```terraform
data "decort_rg_list_vins" "list_vins" {
  #id ресурсной группы
  #обязательный параметр
  #тип - целое число
  rg_id = 123

  #фильтр по id vins
  #опциональный параметр
  #тип - целое число
  #vins_id = 100
 
  #фильтр по имени vins
  #опциональный параметр
  #тип - строка
  #name = "test"

  #id аккаунта для получения списка балансировщиков нагрузки
  #опциональный параметр 
  #тип - целое число
  #account_id = 11111

  #фильтр по IP внешней сети
  #опциональный параметр
  #тип - строка
  #ext_ip = "test"

  #сортировка по одному из поддерживаемых полей
  #опциональный параметр
  #тип - строка
  #формат - "+поле" по возрастанию / "-поле" по убыванию
  #sort_by = "+name"

  #номер страницы для отображения
  #опциональный параметр
  #тип - целое число
  #если не задан - выводятся все доступные данные
  #page = 2

  #размер страницы
  #опциональный параметр
  #тип - целое число
  #если не задан - выводятся все доступные данные
  #size = 3
}
```

Данный пример подразумевает, что ранее по тексту tf-файла выполнена корректная инициализация провайдера (подробнее об инициализации Terraform провайдера DECORT см. соответствующий [раздел](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.7.1/04.02-Инициализация-Terraform-провайдера-DECORT.md) и [пример](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.7.1/02.-Пример-работы.md)). 

После успешного завершения такого вызова **decort_rg_list_vins** в переменной `data.decort_rg_list_vins.list_vins` будут сохранены возвращаемые значения. В частности, для получения идентификатора ресурсной группы следует использовать конструкцию `data.decort_rg_list_vins.list_vins.id`
