Функция доступна в провайдере версии 2.3 и выше.

_Data source_ функция **decort_rg_list** служит для получения информации об имеющихся ресурсных группах.

## Аргументы
_Data source_ функция **decort_rg_list** принимает следующие аргументы:
| Аргумент | Тип | Обязательный | Описание |
| --- | --- | --- | --- | 
| by_id | int | :x: | Фильтр по ID |
| name | string | :x: | Фильтр по названию |
| account_id | int | :x: | Фильтр по ID аккаунта |
| account_name | string | :x: | Фильтр по имени аккаунта |
| created_after | int | :x: | Отдает все рес. группы созданные после определенного времени (unix timstamp) |
| created_before | int | :x: | Отдает все рес. группы созданные до определенного времени (unix timstamp) |
| status | string | :x: | Фильтр по статусу |
| lock_status | string | :x: | Фильтр по локстатусу |
| includedeleted | bool | :x: | Включение удаленных rg в результат поиска |
| sort_by | string | :x: | Фильтр по одному из поддерживаемых полей, формат: +или- название поля |
| page | int | :x: | Номер страниц |
| size | int | :x: | Размер страницы |


## Возвращаемые значения
В случае успешного выполнения _data source_ функция **decort_rg_list** возвращает в указанную при вызове переменную, в поле _items_ массив, каждый элемент которого состоит из следующих параметров:

| Параметр | Тип | Описание |
| --- | --- | --- |
| account_id | int | ID аккаунта - владельца ресурсной группы |
| account_name | string | Имя аккаунта - владельца ресурсной группы |
| compute_features | []string | Доступ к дополнительным функциям управления ВМ |
| acl | []Struct{<br/>explicit bool<br/>guid string<br/>right string<br/>status string<br/>type string<br/>user_group_id string<br/>} | Более подробная информация об аккаунтах, имеющих доступ к ресурсной группе<br/>Явно ли указан доступ<br/>guid аккаунта<br/>Права доступа<br/>Статус аккаунта<br/>Тип аккаунта<br/>Идентификатор пользователя-владельца аккаунта<br/>&nbsp;<br/> |
| cpu_allocation_parameter | string | Параметр выделения CPU |
| cpu_allocation_ratio | float | Отношение выделения CPU |
| created_by | string | Информация о создателе группы |
| created_time | int | Время создания группы |
| def_net_id | int | id сети по-умолчанию |
| def_net_type | string | Тип сети по-умолчанию |
| deleted_by | string | Информация о том, кто удалил группу |
| deleted_time | int | Время удаления группы |
| desc | string | Описание группы |
| gid | int | Grid id |
| guid | int | Внутренний идентификатор ресурсной группы |
| rg_id | int | Идентификатор ресурсной группы |
| lock_status | string | Статус блокировки ресурсной группы |
| name | string | Название ресурсной группы |
| register_computes | bool | Имеются ли зарезервированные выч. мощности |
| resource_limits | Struct{<br/>cu_c float<br/>cu_d float<br/>cu_i float<br/>cu_m float<br/>cu_np float<br/>gpu_units float<br/>} | Ограничения ресурсов<br/>Кол-во ядер<br/>Размер диска, ГБ<br/>Число публичных ip адресов<br/>Размер RAM, МБ<br/>Объем трафика, ГБ<br/>Кол-вол графических ядер<br/>&nbsp;<br/> |
| secret | string | Секрет группы |
| status | string | Статус группы |
| updated_by | string | Кем группа была обновлена в последний раз |
| updated_time | int | Время последнего обновления |
| vins | []int | Список идентификаторов виртуальных сетей |
| vms | []int | Список идентификаторов виртуальных машин (устаревшее) |

## Пример использования
Пример вызова _data source_ функции **decort_rg_list**:
```terraform
data "decort_rg_list" "rg_list" {
  #фильтр по id ресурсной группы
  #опциональный параметр
  #тип - целое число
  #by_id = 100

  #фильтр по имени ресурсной группы
  #опциональный параметр
  #тип - строка
  #name = "test"

  #id аккаунта для получения списка ресурсных групп
  #опциональный параметр 
  #тип - целое число
  #account_id = 11111

  #фильтр по имени аккаунта
  #опциональный параметр
  #тип - строка
  #account_name = "test"

  #фильтр по времени создания (после указанного времени)
  #опциональный параметр
  #тип - целое число
  #created_after = "123"

  #фильтр по времени создания (перед указанным временем)
  #опциональный параметр
  #тип - целое число
  #created_before = "123"

  #фильтр по статусу
  #опциональный параметр
  #тип - строка
  #status = "ENABLED"

  #фильтр по lock status
  #опциональный параметр
  #тип - строка
  #lock_status = "UNLOCKED"

  #отображать удаленные ресурсные группы или нет
  #необязательный параметр
  #тип - булев
  #includedeleted = false

  #сортировка по одному из поддерживаемых полей
  #опциональный параметр
  #тип - строка
  #формат - "+поле" по возрастанию / "-поле" по убыванию
  #sort_by = "+name"
  
  #номер страницы
  #необязательный параметр
  #тип - целое число
  #page = 1
  
  #размер страницы
  #необязательный параметр
  #тип - целое число
  #size = 2
}
```

Данный пример подразумевает, что ранее по тексту tf-файла выполнена корректная инициализация провайдера (подробнее об инициализации Terraform провайдера DECORT см. соответствующий [раздел](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.8.3/04.02-Инициализация-Terraform-провайдера-DECORT.md)). 

После успешного завершения такого вызова **decort_rg_list** в переменной `data.decort_rg_list.rl` будут сохранены возвращаемые значения.
